@@ -14,7 +14,7 @@ Exposes the following endpoints:
 import base64
 import json
 import os
-from collections.abc import Awaitable, Callable
+from collections.abc import Callable
 from typing import Any
 
 from dotenv import load_dotenv
@@ -46,6 +46,7 @@ class AgentRequest(BaseModel):
 
     sessionId: int = Field(..., description="Unique session identifier")
     sessionWalletAddress: str = Field(..., description="Wallet address for the session")
+    jobId: str | None = Field(None, description="Optional job ID for status tracking")
     otherParameters: dict[str, Any] | None = Field(
         None, description="Additional parameters"
     )
@@ -59,9 +60,6 @@ class AgentResponse(BaseModel):
     message: str | None = Field(
         None, description="Success message if operation succeeded"
     )
-    data: dict[str, Any] | None = Field(
-        None, description="Optional data returned by the operation"
-    )
 
 
 class HealthResponse(BaseModel):
@@ -86,6 +84,7 @@ class AgentRequestSchema(BaseModel):
     session_wallet_address: str = Field(
         ..., description="Wallet address for the session"
     )
+    job_id: str | None = Field(None, description="Optional job ID for status tracking")
     other_parameters: dict[str, Any] | None = Field(
         None, description="Additional parameters"
     )
@@ -99,9 +98,6 @@ class AgentResponseSchema(BaseModel):
     message: str | None = Field(
         None, description="Optional message describing the operation result"
     )
-    data: dict[str, Any] | None = Field(
-        None, description="Optional data returned by the operation"
-    )
 
 
 class HealthResponseSchema(BaseModel):
@@ -111,10 +107,10 @@ class HealthResponseSchema(BaseModel):
 
 
 # Type aliases for function contracts
-ExecutionFunctionContract = Callable[[AgentRequest], Awaitable[AgentResponse]]
-StopFunctionContract = Callable[[AgentRequest], Awaitable[AgentResponse]]
-ChatFunctionContract = Callable[[AgentRequest], Awaitable[AgentResponse]]
-HealthCheckFunctionContract = Callable[[], Awaitable[dict[str, Any]]]
+ExecutionFunctionContract = Callable[[AgentRequest], AgentResponse | None]
+StopFunctionContract = Callable[[AgentRequest], AgentResponse | None]
+ChatFunctionContract = Callable[[AgentRequest], AgentResponse | None]
+HealthCheckFunctionContract = Callable[[], dict[str, Any]]
 
 
 class AgentConfigClass(BaseModel):
@@ -151,10 +147,10 @@ class Agent:
         ```python
         from agent_sdk import Agent, AgentRequest, AgentResponse
 
-        async def execution_function(request: AgentRequest) -> AgentResponse:
+        def execution_function(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True, message="Execution completed")
 
-        async def stop_function(request: AgentRequest) -> AgentResponse:
+        def stop_function(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True, message="Agent stopped")
 
         agent = Agent(
@@ -172,6 +168,7 @@ class Agent:
         chat_function: ChatFunctionContract | None = None,
         health_check_function: HealthCheckFunctionContract | None = None,
         config: AgentConfig | None = None,
+        base_url: str | None = None,
         **kwargs: Any,
     ) -> None:
         """
@@ -183,6 +180,7 @@ class Agent:
             chat_function: Optional chat function for interactive messaging
             health_check_function: Optional health check function
             config: Optional AgentConfig object
+            base_url: Optional base URL for SDK operations (used for job status updates)
             **kwargs: Additional config parameters
         """
         # Load environment variables from .env file first
@@ -199,6 +197,9 @@ class Agent:
             health_check_function or self._default_health_check_function
         )
 
+        # Store base URL for SDK operations
+        self.base_url = base_url
+
         # Handle configuration
         config_dict = kwargs
         pyproject_config = get_agent_config_from_pyproject()
@@ -223,25 +224,24 @@ class Agent:
         if FASTAPI_AVAILABLE:
             self._setup_fastapi()
 
-    async def _default_stop_function(self, request: AgentRequest) -> AgentResponse:
+    def _default_stop_function(self, request: AgentRequest) -> AgentResponse:
         """Default stop function"""
         return AgentResponse(
             success=True,
             error=None,
             message=f"Agent stopped for session {request.sessionId}",
-            data=None,
         )
 
-    async def _default_health_check_function(self) -> dict[str, Any]:
+    def _default_health_check_function(self) -> dict[str, Any]:
         """Default health check function - returns dict matching TypeScript format"""
         return {"status": "healthy"}
 
-    async def health_check_function(self) -> dict[str, Any]:
+    def health_check_function(self) -> dict[str, Any]:
         """
         Health check function that returns a dict matching TypeScript format
         """
         # _health_check_function is never None due to initialization with default
-        result = await self._health_check_function()
+        result = self._health_check_function()
         # The result is always a dict since _default_health_check_function returns dict
         # and user-provided functions are expected to return dict as well
         return result
@@ -261,7 +261,7 @@ class Agent:
         self.lambda_handler = Mangum(self.app)
 
         @self.app.get("/")
-        async def root() -> dict[str, Any]:
+        def root() -> dict[str, Any]:
             return {
                 "message": f"{self.config.title} is running",
                 "mode": "local",
@@ -269,13 +269,15 @@ class Agent:
             }
 
         @self.app.get("/health")
-        async def health() -> dict[str, Any]:
-            return await self.health_check_function()
+        def health() -> dict[str, Any]:
+            return self.health_check_function()
 
         @self.app.post("/execute")
-        async def execute_agent(request: AgentRequest) -> AgentResponse:
+        def execute_agent(request: AgentRequest) -> AgentResponse:
             try:
-                result = await self.execution_function(request)
+                result = self._execute_with_job_tracking(
+                    request, self.execution_function
+                )
                 return result
             except Exception as e:
                 raise HTTPException(
@@ -286,10 +288,16 @@ class Agent:
         if self.chat_function:
 
             @self.app.post("/chat")
-            async def chat_agent(request: AgentRequest) -> AgentResponse:
+            def chat_agent(request: AgentRequest) -> AgentResponse:
                 try:
                     assert self.chat_function is not None  # Type narrowing for mypy
-                    result = await self.chat_function(request)
+                    result = self.chat_function(request)
+                    if result is None:
+                        result = AgentResponse(
+                            success=True,
+                            error=None,
+                            message="Chat completed (no explicit return)",
+                        )
                     return result
                 except Exception as e:
                     raise HTTPException(
@@ -297,12 +305,16 @@ class Agent:
                     ) from e
 
         @self.app.post("/stop")
-        async def stop_agent(request: AgentRequest) -> AgentResponse:
+        def stop_agent(request: AgentRequest) -> AgentResponse:
             try:
                 if self.stop_function:
-                    result = await self.stop_function(request)
+                    result = self._execute_with_job_tracking(
+                        request, self.stop_function
+                    )
                 else:
-                    result = await self._default_stop_function(request)
+                    result = self._execute_with_job_tracking(
+                        request, self._default_stop_function
+                    )
                 return result
             except Exception as e:
                 raise HTTPException(
@@ -313,10 +325,12 @@ class Agent:
         if self.stop_function:
 
             @self.app.delete("/")
-            async def stop_agent_delete(request: AgentRequest) -> AgentResponse:
+            def stop_agent_delete(request: AgentRequest) -> AgentResponse:
                 try:
                     assert self.stop_function is not None  # Type narrowing for mypy
-                    result = await self.stop_function(request)
+                    result = self._execute_with_job_tracking(
+                        request, self.stop_function
+                    )
                     return result
                 except Exception as e:
                     raise HTTPException(
@@ -324,19 +338,25 @@ class Agent:
                     ) from e
 
         @self.app.post("/{command}")
-        async def handle_command(
+        def handle_command(
             command: str, request: AgentRequest
         ) -> AgentResponse | dict[str, Any]:
             try:
                 if command == "execute":
-                    result = await self.execution_function(request)
+                    result = self._execute_with_job_tracking(
+                        request, self.execution_function
+                    )
                 elif command == "stop":
                     if self.stop_function:
-                        result = await self.stop_function(request)
+                        result = self._execute_with_job_tracking(
+                            request, self.stop_function
+                        )
                     else:
-                        result = await self._default_stop_function(request)
+                        result = self._execute_with_job_tracking(
+                            request, self._default_stop_function
+                        )
                 elif command == "health":
-                    health_result = await self.health_check_function()
+                    health_result = self.health_check_function()
                     return health_result
                 else:
                     raise HTTPException(
@@ -351,10 +371,10 @@ class Agent:
                 ) from e
 
         @self.app.get("/{command}")
-        async def handle_command_get(command: str) -> dict[str, Any]:
+        def handle_command_get(command: str) -> dict[str, Any]:
             try:
                 if command == "health":
-                    return await self.health_check_function()
+                    return self.health_check_function()
                 else:
                     raise HTTPException(
                         status_code=400, detail=f"Unknown command: {command}"
@@ -366,12 +386,13 @@ class Agent:
                     status_code=500, detail=f"Command error: {str(e)}"
                 ) from e
 
-    async def process_request(
+    def process_request(
         self,
         session_id: int,
         session_wallet_address: str,
         command: str = "execute",
         other_parameters: dict[str, Any] | None = None,
+        job_id: str | None = None,
     ) -> dict[str, Any]:
         """
         Process an agent request
@@ -393,22 +414,30 @@ class Agent:
                 request = AgentRequest(
                     sessionId=session_id,
                     sessionWalletAddress=session_wallet_address,
+                    jobId=job_id,
                     otherParameters=other_parameters,
                 )
-                result = await self.execution_function(request)
+                result = self._execute_with_job_tracking(
+                    request, self.execution_function
+                )
             elif command == "stop":
                 request = AgentRequest(
                     sessionId=session_id,
                     sessionWalletAddress=session_wallet_address,
+                    jobId=job_id,
                     otherParameters=other_parameters,
                 )
                 if self.stop_function:
-                    result = await self.stop_function(request)
+                    result = self._execute_with_job_tracking(
+                        request, self.stop_function
+                    )
                 else:
-                    result = await self._default_stop_function(request)
+                    result = self._execute_with_job_tracking(
+                        request, self._default_stop_function
+                    )
             elif command == "health":
                 # Simply call health_check_function which handles all the conversion
-                return await self.health_check_function()
+                return self.health_check_function()
             else:
                 raise ValueError(f"Unknown command: {command}")
 
@@ -416,12 +445,156 @@ class Agent:
             return result.model_dump()
 
         except Exception as e:
-            self.logger.error(f"Request processing error: {e}")
-            return {
-                "success": False,
-                "error": str(e),
-                "message": "Request processing failed",
+            # Enhanced error logging with more context
+            error_type = type(e).__name__
+            error_message = str(e)
+            self.logger.error(
+                f"Request processing error: {error_type}: {error_message}"
+            )
+            self.logger.error(
+                f"Session ID: {session_id}, Command: {command}, Job ID: {job_id}"
+            )
+
+            # If we have a job_id, report the failure
+            if job_id:
+                try:
+                    self._update_job_status(
+                        session_id, job_id, "failed", f"{error_type}: {error_message}"
+                    )
+                except Exception as status_error:
+                    self.logger.error(f"Failed to update job status: {status_error}")
+
+            # Return a proper AgentResponse for consistency
+            return AgentResponse(
+                success=False,
+                error=f"{error_type}: {error_message}",
+                message="Request processing failed",
+            ).model_dump()
+
+    def _execute_with_job_tracking(
+        self,
+        request: AgentRequest,
+        function: ExecutionFunctionContract | StopFunctionContract,
+    ) -> AgentResponse:
+        """
+        Execute a function with automatic job status tracking.
+
+        Args:
+            request: The agent request containing jobId
+            function: The function to execute (execution or stop)
+
+        Returns:
+            The function result
+        """
+        try:
+            # Execute the function
+            result = function(request)
+
+            # Handle None return (agent didn't return AgentResponse)
+            if result is None:
+                result = AgentResponse(
+                    success=True,
+                    error=None,
+                    message="Agent execution completed (no explicit return)",
+                )
+
+            # If we have a job_id and the function succeeded, update status
+            if request.jobId and result.success:
+                try:
+                    self._update_job_status(request.sessionId, request.jobId, "success")
+                except Exception as status_error:
+                    self.logger.error(
+                        f"Failed to update job status to success: {status_error}"
+                    )
+            elif request.jobId and not result.success:
+                # Function returned failure
+                try:
+                    error_msg = (
+                        result.error or result.message or "Function returned failure"
+                    )
+                    self._update_job_status(
+                        request.sessionId, request.jobId, "failed", error_msg
+                    )
+                except Exception as status_error:
+                    self.logger.error(
+                        f"Failed to update job status to failed: {status_error}"
+                    )
+
+            return result
+
+        except Exception as e:
+            # Function threw an exception - enhanced error logging
+            error_type = type(e).__name__
+            error_message = str(e)
+            self.logger.error(f"Agent function error: {error_type}: {error_message}")
+            self.logger.error(
+                f"Session ID: {request.sessionId}, Job ID: {request.jobId}"
+            )
+
+            if request.jobId:
+                try:
+                    self._update_job_status(
+                        request.sessionId,
+                        request.jobId,
+                        "failed",
+                        f"{error_type}: {error_message}",
+                    )
+                except Exception as status_error:
+                    self.logger.error(
+                        f"Failed to update job status after exception: {status_error}"
+                    )
+
+            # Return a failure response instead of re-raising
+            return AgentResponse(
+                success=False,
+                error=f"{error_type}: {error_message}",
+                message="Agent execution failed due to uncaught exception",
+            )
+
+    def _update_job_status(
+        self,
+        session_id: int,
+        job_id: str,
+        status: str,
+        error_message: str | None = None,
+    ) -> None:
+        """
+        Update job status using the AgentSdk.
+
+        Args:
+            session_id: Session ID for SDK initialization
+            job_id: Job ID to update
+            status: New status ("success" or "failed")
+            error_message: Optional error message for failed status
+        """
+        try:
+            # Import here to avoid circular import
+            from .agent_sdk import AgentSdk
+            from .types.config import SDKConfig
+
+            # Create SDK instance for this session using the same base URL as the agent
+            sdk = AgentSdk(
+                SDKConfig(
+                    session_id=session_id,
+                    verbose=False,
+                    testing=False,
+                    base_url=self.base_url,
+                )
+            )
+
+            # Update job status
+            update_request = {
+                "jobId": job_id,
+                "status": status,
             }
+            if error_message:
+                update_request["errorMessage"] = error_message
+
+            sdk._update_job_status(update_request)
+
+        except Exception as e:
+            self.logger.error(f"Failed to update job status: {e}")
+            # Don't re-raise - job status update failures shouldn't break the main flow
 
     def lambda_handler_func(
         self, event: dict[str, Any], context: Any
@@ -461,6 +634,7 @@ class Agent:
             # Extract required parameters
             session_id = body.get("sessionId")
             session_wallet_address = body.get("sessionWalletAddress")
+            job_id = body.get("jobId")
             other_parameters = body.get("otherParameters", {})
 
             # Single info log with all relevant information
@@ -481,19 +655,19 @@ class Agent:
                     headers={},
                 ).model_dump()
 
-            # Process the request (this needs to be sync for Lambda)
-            import asyncio
-
+            # Process the request
             if command == "health":
                 # Health check - no sessionId/walletAddress needed
-                result: dict[str, Any] = asyncio.run(self.health_check_function())
+                result: dict[str, Any] = self.health_check_function()
             else:
                 if session_id is None or session_wallet_address is None:
                     raise ValueError("Missing required parameters")
-                result = asyncio.run(
-                    self.process_request(
-                        session_id, session_wallet_address, command, other_parameters
-                    )
+                result = self.process_request(
+                    session_id,
+                    session_wallet_address,
+                    command,
+                    other_parameters,
+                    job_id,
                 )
 
             self.logger.info(
@@ -502,10 +676,30 @@ class Agent:
 
             # If the result indicates failure, return 500 status code
             if isinstance(result, dict) and result.get("success") is False:
+                error_message = result.get("error", "Unknown error")
+                # Ensure error message is JSON-safe
+                try:
+                    error_body = json.dumps(
+                        {
+                            "success": False,
+                            "error": error_message,
+                            "message": "Agent execution failed",
+                        }
+                    )
+                except (TypeError, ValueError):
+                    # Fallback if error message contains non-JSON-serializable content
+                    error_body = json.dumps(
+                        {
+                            "success": False,
+                            "error": str(error_message),
+                            "message": "Agent execution failed",
+                        }
+                    )
+
                 return LambdaResponse(
                     statusCode=500,
-                    body=f"Internal server error: {result.get('error', 'Unknown error')}",
-                    headers={},
+                    body=error_body,
+                    headers={"Content-Type": "application/json"},
                 ).model_dump()
 
             return LambdaResponse(
@@ -631,6 +825,7 @@ def create_agent_handler(
     chat_function: ChatFunctionContract | None = None,
     stop_function: StopFunctionContract | None = None,
     health_check_function: HealthCheckFunctionContract | None = None,
+    base_url: str | None = None,
 ) -> Agent:
     """
     Convenience factory to create an Agent from function handlers.
@@ -640,6 +835,7 @@ def create_agent_handler(
         chat_function: Optional chat function for handling interactive messaging
         stop_function: Optional winddown function for cleanup operations
         health_check_function: Optional health check function for monitoring agent health status
+        base_url: Optional base URL for SDK operations (used for job status updates)
 
     Returns:
         Agent instance configured with the provided functions
@@ -648,10 +844,10 @@ def create_agent_handler(
         ```python
         from agent_sdk import create_agent_handler, AgentRequest, AgentResponse
 
-        async def execution_function(request: AgentRequest) -> AgentResponse:
+        def execution_function(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True, message="Execution completed")
 
-        agent = create_agent_handler(execution_function)
+        agent = create_agent_handler(execution_function, base_url="http://localhost:4001")
         agent.run()
         ```
     """
@@ -660,4 +856,5 @@ def create_agent_handler(
         chat_function=chat_function,
         stop_function=stop_function,
         health_check_function=health_check_function,
+        base_url=base_url,
     )
