@@ -19,7 +19,7 @@ class TestAgent:
     def mock_execution_function(self):
         """Mock execution function for testing"""
 
-        async def mock_func(request: AgentRequest) -> AgentResponse:
+        def mock_func(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True, data={"message": "test"})
 
         return mock_func
@@ -28,8 +28,8 @@ class TestAgent:
     def mock_stop_function(self):
         """Mock stop function for testing"""
 
-        async def mock_func(request: AgentRequest) -> AgentResponse:
-            return AgentResponse(success=True, data={"cleanup": "done"})
+        def mock_func(request: AgentRequest) -> AgentResponse:
+            return AgentResponse(success=True, message="Cleanup completed")
 
         return mock_func
 
@@ -37,7 +37,7 @@ class TestAgent:
     def mock_health_function(self):
         """Mock health function for testing"""
 
-        async def mock_func():
+        def mock_func():
             return {"status": "healthy", "version": "1.0.0"}
 
         return mock_func
@@ -100,8 +100,7 @@ class TestAgent:
         with pytest.raises(TypeError):
             Agent()
 
-    @pytest.mark.asyncio
-    async def test_process_request_success(
+    def test_process_request_success(
         self, mock_execution_function, mock_stop_function, sample_request
     ):
         """Test successful request processing"""
@@ -109,27 +108,25 @@ class TestAgent:
             execution_function=mock_execution_function, stop_function=mock_stop_function
         )
 
-        result = await agent.process_request(
+        result = agent.process_request(
             session_id=sample_request.sessionId,
             session_wallet_address=sample_request.sessionWalletAddress,
             command="execute",
         )
 
         assert result["success"] is True
-        assert "data" in result
 
-    @pytest.mark.asyncio
-    async def test_process_request_with_error(self, mock_stop_function, sample_request):
+    def test_process_request_with_error(self, mock_stop_function, sample_request):
         """Test request processing with error"""
 
-        async def error_function(request: AgentRequest) -> AgentResponse:
+        def error_function(request: AgentRequest) -> AgentResponse:
             raise Exception("Test error")
 
         agent = Agent(
             execution_function=error_function, stop_function=mock_stop_function
         )
 
-        result = await agent.process_request(
+        result = agent.process_request(
             session_id=sample_request.sessionId,
             session_wallet_address=sample_request.sessionWalletAddress,
             command="execute",
@@ -139,28 +136,26 @@ class TestAgent:
         assert "error" in result
         assert "Test error" in result["error"]
 
-    @pytest.mark.asyncio
-    async def test_default_stop_function(self, mock_stop_function, sample_request):
+    def test_default_stop_function(self, mock_stop_function, sample_request):
         """Test the default stop function"""
         agent = Agent(
             execution_function=lambda req: AgentResponse(success=True),
             stop_function=mock_stop_function,
         )
 
-        result = await agent.stop_function(sample_request)
+        result = agent.stop_function(sample_request)
 
         assert isinstance(result, AgentResponse)
         assert result.success is True
 
-    @pytest.mark.asyncio
-    async def test_default_health_function(self, mock_stop_function):
+    def test_default_health_function(self, mock_stop_function):
         """Test the default health function"""
         agent = Agent(
             execution_function=lambda req: AgentResponse(success=True),
             stop_function=mock_stop_function,
         )
 
-        result = await agent.health_check_function()
+        result = agent.health_check_function()
 
         assert isinstance(result, dict)
         assert "status" in result
@@ -231,33 +226,31 @@ class TestAgent:
 class TestAgentIntegration:
     """Integration tests for the Agent class"""
 
-    @pytest.mark.asyncio
-    async def test_full_agent_workflow(self):
+    def test_full_agent_workflow(self):
         """Test a complete agent workflow"""
 
-        async def execution_func(request: AgentRequest) -> AgentResponse:
+        def execution_func(request: AgentRequest) -> AgentResponse:
             return AgentResponse(
-                success=True, data={"processed": True, "session": request.sessionId}
+                success=True, message=f"Processed session {request.sessionId}"
             )
 
-        async def stop_func(request: AgentRequest) -> AgentResponse:
-            return AgentResponse(success=True, data={"cleaned_up": True})
+        def stop_func(request: AgentRequest) -> AgentResponse:
+            return AgentResponse(success=True, message="Cleaned up successfully")
 
         agent = Agent(execution_function=execution_func, stop_function=stop_func)
 
         # Test execution
-        exec_result = await agent.process_request(
+        exec_result = agent.process_request(
             session_id=123,
             session_wallet_address="0x1234567890abcdef",
             command="execute",
         )
 
         assert exec_result["success"] is True
-        assert exec_result["data"]["processed"] is True
-        assert exec_result["data"]["session"] == 123
+        assert "Processed session 123" in exec_result["message"]
 
         # Test stop
-        stop_result = await agent.stop_function(
+        stop_result = agent.stop_function(
             AgentRequest(
                 sessionId=123,
                 sessionWalletAddress="0x1234567890abcdef",
@@ -266,7 +259,7 @@ class TestAgentIntegration:
         )
 
         assert stop_result.success is True
-        assert stop_result.data["cleaned_up"] is True
+        assert "Cleaned up successfully" in stop_result.message
 
 
 class TestAgentLambdaHandler:
@@ -276,7 +269,7 @@ class TestAgentLambdaHandler:
     def mock_execution_function(self):
         """Mock execution function for testing"""
 
-        async def mock_func(request: AgentRequest) -> AgentResponse:
+        def mock_func(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True, data={"message": "test"})
 
         return mock_func
@@ -285,8 +278,8 @@ class TestAgentLambdaHandler:
     def mock_stop_function(self):
         """Mock stop function for testing"""
 
-        async def mock_func(request: AgentRequest) -> AgentResponse:
-            return AgentResponse(success=True, data={"cleanup": "done"})
+        def mock_func(request: AgentRequest) -> AgentResponse:
+            return AgentResponse(success=True, message="Cleanup completed")
 
         return mock_func
 
@@ -294,7 +287,7 @@ class TestAgentLambdaHandler:
     def mock_health_function(self):
         """Mock health function for testing"""
 
-        async def mock_func():
+        def mock_func():
             return {"status": "healthy", "version": "1.0.0"}
 
         return mock_func
@@ -402,7 +395,7 @@ class TestAgentLambdaHandler:
     def test_lambda_handler_error_handling(self, mock_stop_function):
         """Test Lambda handler error handling"""
 
-        async def error_function(request: AgentRequest) -> AgentResponse:
+        def error_function(request: AgentRequest) -> AgentResponse:
             raise Exception("Test error")
 
         agent = Agent(
@@ -417,7 +410,7 @@ class TestAgentLambdaHandler:
         result = agent.lambda_handler_func(event, {})
 
         assert result["statusCode"] == 500
-        assert "Internal server error" in result["body"]
+        assert "Exception: Test error" in result["body"]
 
     def test_lambda_handler_delete_stop_method(
         self, mock_execution_function, mock_stop_function
@@ -529,27 +522,26 @@ class TestAgentCommandProcessing:
 
     @pytest.fixture
     def mock_execution_function(self):
-        async def mock_func(request: AgentRequest) -> AgentResponse:
+        def mock_func(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True, data={"executed": True})
 
         return mock_func
 
     @pytest.fixture
     def mock_stop_function(self):
-        async def mock_func(request: AgentRequest) -> AgentResponse:
-            return AgentResponse(success=True, data={"stopped": True})
+        def mock_func(request: AgentRequest) -> AgentResponse:
+            return AgentResponse(success=True, message="Agent stopped")
 
         return mock_func
 
     @pytest.fixture
     def mock_health_function(self):
-        async def mock_func():
+        def mock_func():
             return {"status": "healthy"}
 
         return mock_func
 
-    @pytest.mark.asyncio
-    async def test_process_request_stop_command(
+    def test_process_request_stop_command(
         self, mock_execution_function, mock_stop_function
     ):
         """Test process_request with stop command"""
@@ -557,13 +549,12 @@ class TestAgentCommandProcessing:
             execution_function=mock_execution_function, stop_function=mock_stop_function
         )
 
-        result = await agent.process_request(123, "0x123", "stop")
+        result = agent.process_request(123, "0x123", "stop")
 
         assert result["success"] is True
-        assert result["data"]["stopped"] is True
+        assert "Agent stopped" in result["message"]
 
-    @pytest.mark.asyncio
-    async def test_process_request_health_command(
+    def test_process_request_health_command(
         self, mock_execution_function, mock_stop_function, mock_health_function
     ):
         """Test process_request with health command"""
@@ -573,12 +564,11 @@ class TestAgentCommandProcessing:
             health_check_function=mock_health_function,
         )
 
-        result = await agent.process_request(123, "0x123", "health")
+        result = agent.process_request(123, "0x123", "health")
 
         assert result["status"] == "healthy"
 
-    @pytest.mark.asyncio
-    async def test_process_request_unknown_command(
+    def test_process_request_unknown_command(
         self, mock_execution_function, mock_stop_function
     ):
         """Test process_request with unknown command"""
@@ -586,13 +576,12 @@ class TestAgentCommandProcessing:
             execution_function=mock_execution_function, stop_function=mock_stop_function
         )
 
-        result = await agent.process_request(123, "0x123", "unknown")
+        result = agent.process_request(123, "0x123", "unknown")
 
         assert result["success"] is False
         assert "Unknown command" in result["error"]
 
-    @pytest.mark.asyncio
-    async def test_process_request_health_without_function(
+    def test_process_request_health_without_function(
         self, mock_execution_function, mock_stop_function
     ):
         """Test process_request health command without health function"""
@@ -601,7 +590,7 @@ class TestAgentCommandProcessing:
         )
         agent._health_function = None
 
-        result = await agent.process_request(123, "0x123", "health")
+        result = agent.process_request(123, "0x123", "health")
 
         # Should still return healthy since health_function method handles None case
         assert isinstance(result, dict)
@@ -613,14 +602,14 @@ class TestAgentRunMethod:
 
     @pytest.fixture
     def mock_execution_function(self):
-        async def mock_func(request: AgentRequest) -> AgentResponse:
+        def mock_func(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True)
 
         return mock_func
 
     @pytest.fixture
     def mock_stop_function(self):
-        async def mock_func(request: AgentRequest) -> AgentResponse:
+        def mock_func(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True)
 
         return mock_func
@@ -666,7 +655,7 @@ class TestAgentErrorHandling:
 
     @pytest.fixture
     def mock_stop_function(self):
-        async def mock_func(request: AgentRequest) -> AgentResponse:
+        def mock_func(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True)
 
         return mock_func
@@ -676,11 +665,10 @@ class TestAgentErrorHandling:
         with pytest.raises(ValueError, match="execution_function is required"):
             Agent(execution_function=None, stop_function=mock_stop_function)
 
-    @pytest.mark.asyncio
-    async def test_default_stop_function_usage(self, mock_stop_function):
+    def test_default_stop_function_usage(self, mock_stop_function):
         """Test that default stop function works"""
 
-        async def mock_exec(request: AgentRequest) -> AgentResponse:
+        def mock_exec(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True)
 
         agent = Agent(execution_function=mock_exec, stop_function=mock_stop_function)
@@ -689,14 +677,14 @@ class TestAgentErrorHandling:
         request = AgentRequest(
             sessionId=123, sessionWalletAddress="0x123", otherParameters={}
         )
-        result = await agent.stop_function(request)
+        result = agent.stop_function(request)
 
         assert result.success is True
 
     def test_get_worker_export_without_fastapi(self, mock_stop_function):
         """Test get_worker_export when FastAPI is not available"""
 
-        async def mock_exec(request: AgentRequest) -> AgentResponse:
+        def mock_exec(request: AgentRequest) -> AgentResponse:
             return AgentResponse(success=True)
 
         with patch("agent_sdk.agent.FASTAPI_AVAILABLE", False):
